package gov.cms.fiss.pricers.ltch.core;

import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.tables.DataTables;
import java.math.BigDecimal;
import java.math.RoundingMode;

public class Ltch2020PricerContext extends LtchPricerContext {
  public static final String CALCULATION_VERSION = "V20.2";
  private static final BigDecimal STANDARD_QUALITY_FED_RATE_2020 = new BigDecimal("42677.64");
  private static final BigDecimal STANDARD_FED_RATE_2020 = new BigDecimal("41844.90");
  private static final BigDecimal NATIONAL_LABOR_PERCENT_2020 = new BigDecimal(".66300");
  private static final BigDecimal NATIONAL_NON_LABOR_PERCENT_2020 = new BigDecimal(".33700");
  private static final BigDecimal HIGH_IPPS_NATIONAL_LABOR_SHARE_2020 = new BigDecimal("3959.10");
  private static final BigDecimal LOW_IPPS_NATIONAL_LABOR_SHARE_2020 = new BigDecimal("3593.91");
  private static final BigDecimal HIGH_IPPS_NATIONAL_NON_LABOR_SHARE_2020 =
      new BigDecimal("1837.53");
  private static final BigDecimal LOW_IPPS_NATIONAL_NON_LABOR_SHARE_2020 =
      new BigDecimal("2202.72");

  public Ltch2020PricerContext(LtchInput input, LtchOutput output, DataTables dataTables) {
    super(input, output, dataTables);
    getProspectivePaymentRecord()
        .setBudgetNeutralRate(BigDecimal.ONE.setScale(3, RoundingMode.HALF_UP));
    this.calculationVersion = CALCULATION_VERSION;
  }

  @Override
  public BigDecimal getStandardQualityFedRate() {
    return STANDARD_QUALITY_FED_RATE_2020;
  }

  @Override
  public BigDecimal getStandardFedRate() {
    return STANDARD_FED_RATE_2020;
  }

  @Override
  public BigDecimal getNationalLaborPercent() {
    return NATIONAL_LABOR_PERCENT_2020;
  }

  @Override
  public BigDecimal getNationalNonLaborPercent() {
    return NATIONAL_NON_LABOR_PERCENT_2020;
  }

  @Override
  public BigDecimal getHighIppsNationalLaborShare() {
    return HIGH_IPPS_NATIONAL_LABOR_SHARE_2020;
  }

  @Override
  public BigDecimal getLowIppsNationalLaborShare() {
    return LOW_IPPS_NATIONAL_LABOR_SHARE_2020;
  }

  @Override
  public BigDecimal getHighIppsNationalNonLaborShare() {
    return HIGH_IPPS_NATIONAL_NON_LABOR_SHARE_2020;
  }

  @Override
  public BigDecimal getLowIppsNationalNonLaborShare() {
    return LOW_IPPS_NATIONAL_NON_LABOR_SHARE_2020;
  }
}
