package gov.cms.fiss.pricers.ltch.core;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.rules.AssemblePpsVariables;
import gov.cms.fiss.pricers.ltch.core.rules.AssignDates;
import gov.cms.fiss.pricers.ltch.core.rules.AssignFloorCBSA;
import gov.cms.fiss.pricers.ltch.core.rules.CalcFinalPayment;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateChargeThreshold;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateFacilityCosts;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateHighCostOutlier;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateHighCostOutlierBlend;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateIppsComparablePayment;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateShortStay;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateShortStayBlendedPayment;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateSiteNeutralPayment;
import gov.cms.fiss.pricers.ltch.core.rules.CalculateStandardPayment;
import gov.cms.fiss.pricers.ltch.core.rules.CleanupPaymentSetting;
import gov.cms.fiss.pricers.ltch.core.rules.CleanupShortStay;
import gov.cms.fiss.pricers.ltch.core.rules.CleanupShortStayBlendedPayment;
import gov.cms.fiss.pricers.ltch.core.rules.CleanupStandardPayment;
import gov.cms.fiss.pricers.ltch.core.rules.CreateNewProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.rules.DetermineLtchWageIndex;
import gov.cms.fiss.pricers.ltch.core.rules.DeterminePaymentType;
import gov.cms.fiss.pricers.ltch.core.rules.EditDrgCode;
import gov.cms.fiss.pricers.ltch.core.rules.EditInputData;
import gov.cms.fiss.pricers.ltch.core.rules.EditIppsDrgCode;
import gov.cms.fiss.pricers.ltch.core.rules.FinalizeClaim;
import gov.cms.fiss.pricers.ltch.core.rules.SetErrorProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.rules.SetFinalReturnCodes;
import gov.cms.fiss.pricers.ltch.core.rules.SetNewReturnCodes;
import gov.cms.fiss.pricers.ltch.core.rules.SetOldReturnCodes;
import gov.cms.fiss.pricers.ltch.core.rules.SetSiteNeutralCodes;
import gov.cms.fiss.pricers.ltch.core.rules.rules2020.DetermineIppsWageIndex;
import gov.cms.fiss.pricers.ltch.core.tables.DataTables;
import java.util.List;

public class Ltch2020RulePricer extends LtchRulePricer {

  private static final List<CalculationRule<LtchInput, LtchOutput, LtchPricerContext>> RULES =
      rules();

  public Ltch2020RulePricer(DataTables dataTables) {
    super(dataTables, RULES);
  }

  private static List<CalculationRule<LtchInput, LtchOutput, LtchPricerContext>> rules() {
    return List.of(
        new AssignDates(),
        new CreateNewProspectivePaymentRecord(),
        new DetermineLtchWageIndex(),
        new DetermineIppsWageIndex(),
        new EditInputData(),
        new EditDrgCode(),
        new EditIppsDrgCode(),
        new AssignFloorCBSA(),
        new AssemblePpsVariables(),
        new CalculateFacilityCosts(),
        new DeterminePaymentType(),
        // TODO refactor calculate rules to use parent rules and check payment types
        new CalculateStandardPayment(),
        // TODO refactor shortstay rules to a list of rules for one condition check
        new CalculateShortStay(),
        new CalculateShortStayBlendedPayment(),
        new CalculateIppsComparablePayment(),
        new CleanupShortStayBlendedPayment(),
        new CleanupShortStay(),
        new CleanupPaymentSetting(),
        new CleanupStandardPayment(),
        new CalculateIppsComparablePayment(),
        new SetSiteNeutralCodes(),
        new CalculateHighCostOutlier(),
        new CalculateHighCostOutlierBlend(),
        new CalculateChargeThreshold(),
        new CalculateSiteNeutralPayment(),
        // TODO this stuff below
        // new CalculateHighCostOutlier(
        // List.of(
        //  new CalculateHighCostOutlierNonBlend(),
        //  new CalculateHighCostOutlierBlend())
        // ),
        new CalculateHighCostOutlier(),
        new CalculateHighCostOutlierBlend(),
        new CalculateChargeThreshold(),
        new SetFinalReturnCodes(List.of(new SetOldReturnCodes(), new SetNewReturnCodes())),
        new CalcFinalPayment(),
        new SetErrorProspectivePaymentRecord(),
        new FinalizeClaim());
  }

  @Override
  protected LtchPricerContext contextFor(LtchInput input) {
    LtchOutput output = new LtchOutput(new ProspectivePaymentRecord());
    return new Ltch2020PricerContext(input, output, dataTables);
  }
}
