package gov.cms.fiss.pricers.ltch.core;

import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.tables.DataTables;
import java.math.BigDecimal;
import java.math.RoundingMode;

public class Ltch2021PricerContext extends LtchPricerContext {

  private static final BigDecimal STANDARD_QUALITY_FED_RATE_2021 = new BigDecimal("43755.34");
  private static final BigDecimal STANDARD_FED_RATE_2021 = new BigDecimal("42899.90");
  private static final BigDecimal NATIONAL_LABOR_PERCENT_2021 = new BigDecimal(".68100");
  private static final BigDecimal NATIONAL_NON_LABOR_PERCENT_2021 = new BigDecimal(".31900");
  private static final BigDecimal HIGH_IPPS_NATIONAL_LABOR_SHARE_2021 = new BigDecimal("4071.57");
  private static final BigDecimal LOW_IPPS_NATIONAL_LABOR_SHARE_2021 = new BigDecimal("3696.01");
  private static final BigDecimal HIGH_IPPS_NATIONAL_NON_LABOR_SHARE_2021 =
      new BigDecimal("1889.74");
  private static final BigDecimal LOW_IPPS_NATIONAL_NON_LABOR_SHARE_2021 =
      new BigDecimal("2265.30");

  public Ltch2021PricerContext(LtchInput input, LtchOutput output, DataTables dataTables) {
    super(input, output, dataTables);
    calculationVersion = "V21.2";
    setHoldFixedLossAmountStandard(new BigDecimal("27195.00").setScale(2, RoundingMode.HALF_UP));
    setHoldFixedLossAmountSiteNeutral(new BigDecimal("29064.00").setScale(2, RoundingMode.HALF_UP));
    setHoldIppsCapitalStandardFedRate(new BigDecimal("466.21").setScale(2, RoundingMode.HALF_UP));
    setHoldOperatingDshReductionFactor(new BigDecimal("0.7965").setScale(4, RoundingMode.HALF_UP));
    getProspectivePaymentRecord()
        .setBudgetNeutralRate(BigDecimal.ONE.setScale(3, RoundingMode.HALF_UP));
  }

  @Override
  public BigDecimal getStandardQualityFedRate() {
    return STANDARD_QUALITY_FED_RATE_2021;
  }

  @Override
  public BigDecimal getStandardFedRate() {
    return STANDARD_FED_RATE_2021;
  }

  @Override
  public BigDecimal getNationalLaborPercent() {
    return NATIONAL_LABOR_PERCENT_2021;
  }

  @Override
  public BigDecimal getNationalNonLaborPercent() {
    return NATIONAL_NON_LABOR_PERCENT_2021;
  }

  @Override
  public BigDecimal getHighIppsNationalLaborShare() {
    return HIGH_IPPS_NATIONAL_LABOR_SHARE_2021;
  }

  @Override
  public BigDecimal getLowIppsNationalLaborShare() {
    return LOW_IPPS_NATIONAL_LABOR_SHARE_2021;
  }

  @Override
  public BigDecimal getHighIppsNationalNonLaborShare() {
    return HIGH_IPPS_NATIONAL_NON_LABOR_SHARE_2021;
  }

  @Override
  public BigDecimal getLowIppsNationalNonLaborShare() {
    return LOW_IPPS_NATIONAL_NON_LABOR_SHARE_2021;
  }
}
