package gov.cms.fiss.pricers.ltch.core;

import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.tables.DataTables;
import java.math.BigDecimal;
import java.math.RoundingMode;

public class Ltch2022PricerContext extends LtchPricerContext {

  private static final BigDecimal STANDARD_QUALITY_FED_RATE_2022 = new BigDecimal("44713.67");
  private static final BigDecimal STANDARD_FED_RATE_2022 = new BigDecimal("43836.08");
  private static final BigDecimal NATIONAL_LABOR_PERCENT_2022 = new BigDecimal(".67900");
  private static final BigDecimal NATIONAL_NON_LABOR_PERCENT_2022 = new BigDecimal(".32100");
  private static final BigDecimal HIGH_IPPS_NATIONAL_LABOR_SHARE_2022 = new BigDecimal("4138.24");
  private static final BigDecimal LOW_IPPS_NATIONAL_LABOR_SHARE_2022 = new BigDecimal("3795.42");
  private static final BigDecimal HIGH_IPPS_NATIONAL_NON_LABOR_SHARE_2022 =
      new BigDecimal("1983.41");
  private static final BigDecimal LOW_IPPS_NATIONAL_NON_LABOR_SHARE_2022 =
      new BigDecimal("2326.23");

  public Ltch2022PricerContext(LtchInput input, LtchOutput output, DataTables dataTables) {
    super(input, output, dataTables);
    calculationVersion = "V22.0";
    setHoldFixedLossAmountStandard(new BigDecimal("33015.00").setScale(2, RoundingMode.HALF_UP));
    setHoldFixedLossAmountSiteNeutral(new BigDecimal("30988.00").setScale(2, RoundingMode.HALF_UP));
    setHoldIppsCapitalStandardFedRate(new BigDecimal("472.59").setScale(2, RoundingMode.HALF_UP));
    setHoldOperatingDshReductionFactor(new BigDecimal("0.7643").setScale(4, RoundingMode.HALF_UP));
    getProspectivePaymentRecord()
        .setBudgetNeutralRate(BigDecimal.ONE.setScale(3, RoundingMode.HALF_UP));
  }

  /** Getters for annual constants, often called 'rates,' that may change each year */
  @Override
  public BigDecimal getStandardQualityFedRate() {
    return STANDARD_QUALITY_FED_RATE_2022;
  }

  @Override
  public BigDecimal getStandardFedRate() {
    return STANDARD_FED_RATE_2022;
  }

  @Override
  public BigDecimal getNationalLaborPercent() {
    return NATIONAL_LABOR_PERCENT_2022;
  }

  @Override
  public BigDecimal getNationalNonLaborPercent() {
    return NATIONAL_NON_LABOR_PERCENT_2022;
  }

  @Override
  public BigDecimal getHighIppsNationalLaborShare() {
    return HIGH_IPPS_NATIONAL_LABOR_SHARE_2022;
  }

  @Override
  public BigDecimal getLowIppsNationalLaborShare() {
    return LOW_IPPS_NATIONAL_LABOR_SHARE_2022;
  }

  @Override
  public BigDecimal getHighIppsNationalNonLaborShare() {
    return HIGH_IPPS_NATIONAL_NON_LABOR_SHARE_2022;
  }

  @Override
  public BigDecimal getLowIppsNationalNonLaborShare() {
    return LOW_IPPS_NATIONAL_NON_LABOR_SHARE_2022;
  }
}
