package gov.cms.fiss.pricers.ltch.core;

import gov.cms.fiss.pricers.common.api.InternalPricerException;
import gov.cms.fiss.pricers.common.api.YearNotImplementedException;
import gov.cms.fiss.pricers.common.application.ClaimProcessor;
import gov.cms.fiss.pricers.common.application.PricerDispatch;
import gov.cms.fiss.pricers.ltch.LtchPricerConfiguration;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.codes.ErrorCode;
import gov.cms.fiss.pricers.ltch.core.tables.DataTables;

public class LtchPricerDispatch
    extends PricerDispatch<LtchInput, LtchOutput, LtchPricerConfiguration> {

  public LtchPricerDispatch(LtchPricerConfiguration pricerConfiguration) {
    super(pricerConfiguration, o -> o.getOutputRecord().getRtc());
  }

  @Override
  protected void initializeReferences(LtchPricerConfiguration pricerConfiguration) {
    for (final int supportedYear : pricerConfiguration.getSupportedYears()) {
      switch (supportedYear) {
        case 2020:
          yearReference.register(
              supportedYear, Ltch2020RulePricer.class, DataTables.forYear(supportedYear));
          break;
        case 2021:
          yearReference.register(
              supportedYear, Ltch2021RulePricer.class, DataTables.forYear(supportedYear));
          break;
        case 2022:
          yearReference.register(
              supportedYear, Ltch2022RulePricer.class, DataTables.forYear(supportedYear));
          break;
        default:
          break;
      }
    }
  }

  @Override
  protected ClaimProcessor<LtchInput, LtchOutput> getProcessor(LtchInput input)
      throws YearNotImplementedException, InternalPricerException {
    return yearReference.fromFiscalYear(
        input.getBillingRecord().getDischargeDate(), "dischargeDate");
  }

  @Override
  protected boolean isErrorOutput(LtchOutput output) {
    return ErrorCode.isErrorCode(output.getOutputRecord().getRtc());
  }
}
