// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import java.math.BigDecimal;
import java.math.RoundingMode;
import org.apache.commons.lang3.StringUtils;
/* ************************************************************
     GET THE PROVIDER SPECIFIC VARIABLES AND LTCH WAGE INDEX  *
                                                              *
     THE APPROPRIATE SET OF THESE PPS VARIABLES ARE SELECTED  *
     DEPENDING ON THE BILL DISCHARGE DATE AND EFFECTIVE DATE  *
     OF THAT VARIABLE.                                        *
                                                              *
************************************************************** */
/**
 * Converted from 2000-ASSEMBLE-PPS-VARIABLES in COBOL.
 */
public class AssemblePpsVariables implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    // TODO continue here next time
    final gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      final gov.cms.fiss.pricers.ltch.core.models.LtchWageIndexTableEntry ltchWageIndexTableEntry = calculationContext.getLtchWageIndexTableEntry();
      // ASSIGN FULL (5/5) LTCH WAGE INDEX TO ALL CLAIMS DISCHARGED
      // ON AND AFTER 7/1/2008 (THIRD COLUMN WAGE INDEX IN LTWIX***)
      final java.math.BigDecimal ltchWageIndex3 = ltchWageIndexTableEntry.getLtchWageIndex3();
      if (ltchWageIndex3 != null && BigDecimalUtils.isGreaterThanZero(ltchWageIndex3)) {
        prospectivePaymentRecord.setWageIndex(ltchWageIndex3);
      } else {
        prospectivePaymentRecord.setRtc("52");
        return;
      }
      assembleBlendVariables(prospectivePaymentRecord, calculationContext);
    }
  }

  private void assembleBlendVariables(ProspectivePaymentRecord prospectivePaymentRecord, LtchPricerContext calculationContext) {
    final gov.cms.fiss.pricers.ltch.api.BillingRecord billingRecord = calculationContext.getBillingRecord();
    final gov.cms.fiss.pricers.ltch.api.ProviderRecord providerRecord = calculationContext.getProviderRecord();
    int blendYear = Integer.parseInt(providerRecord.getFederalPpsBlendIndicator());
    // OLD POLICY CLAIMS MUST HAVE A BLEND YEAR INDICATOR OF 5
    if (billingRecord.getReviewCode() == 0) {
      blendYear = 5;
    }
    // NEW POLICY CLAIMS MUST HAVE A BLEND YR. IND. OF 6, 7, OR 8
    prospectivePaymentRecord.setBlendYear(blendYear);
    if (billingRecord.getReviewCode() >= 1 && billingRecord.getReviewCode() <= 8 && (blendYear > 8 || blendYear < 6)) {
      prospectivePaymentRecord.setRtc("72");
      return;
    }
    // SET DEFAULT BLEND VARIABLE VALUES
    // H-BLEND-STD = % STANDARD PMT CONTRIBUTES TO FINAL PMT
    // H-BLEND-SNT = % SITE NEUTRAL PMT CONTRIBUTES TO FINAL PMT
    calculationContext.setHoldBlendSiteNeutral(BigDecimal.ZERO);
    calculationContext.setHoldBlendStandard(BigDecimal.ONE.setScale(1, RoundingMode.HALF_UP));
    calculationContext.setHoldBlendRtc(BigDecimal.ZERO);
    // *-------------------------------------------------------------*
    // * FORCE COLA VALUE TO 1.000 (EXCEPT ALASKA & HAWAII)          *
    // *-------------------------------------------------------------*
    if (StringUtils.equalsAny(providerRecord.getNewStateCode(), "02", "12")) {
      prospectivePaymentRecord.setCostOfLivingAdjustment(providerRecord.getCostOfLivingAdjustment());
    } else {
      prospectivePaymentRecord.setCostOfLivingAdjustment(BigDecimal.ONE.setScale(3, RoundingMode.HALF_UP));
    }
  }
}
