// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.api.ProviderRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.codes.ErrorCode;
import gov.cms.fiss.pricers.ltch.core.codes.PaymentType;
import java.math.BigDecimal;
import java.math.RoundingMode;

public class CalculateChargeThreshold implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  private static final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(CalculateChargeThreshold.class);

  /**
   * <pre>
   * ***************************************************************
   * *   CALCULATE CHARGE THRESHOLD & SET ERROR RTC WHEN APPLICABLE*
   * ***************************************************************
   * </pre>
   *
   * Converted from {@code 6100-CALC-CHARGE-THRESHOLD} in the COBOL code.
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    PaymentType paymentType = calculationContext.getPaymentType();
    BillingRecord billingRecord = calculationContext.getBillingRecord();
    ProviderRecord providerRecord = calculationContext.getProviderRecord();
    BigDecimal coveredDays = new BigDecimal(billingRecord.getCoveredDays());
    // IF (PPS-OUTLIER-PAY-AMT > 0 AND
    //     NOT (PMT-BLEND AND H-OUTLIER-PAY-AMT-SNT = 0)) OR
    //    PPS-COT-IND = 'Y'
    if (isOutlierAndSuccessful(calculationContext, prospectivePaymentRecord) && isNonBlendNonSiteNeutralOutlierPaymentOrChangeOfTherapy(calculationContext, prospectivePaymentRecord, paymentType)) {
      // IF B-COV-DAYS < H-LOS OR
      //    (PPS-COT-IND = 'Y' AND P-NEW-OPER-CSTCHG-RATIO NOT = 0)
      if ((BigDecimalUtils.isLessThan(coveredDays, calculationContext.getHoldLengthOfStay()) || ("Y".equals(prospectivePaymentRecord.getChangeOfTherapyIndicator()) && !BigDecimalUtils.isZero(providerRecord.getCostToChargeRatio())))) {
        // COMPUTE PPS-CHRG-THRESHOLD ROUNDED =
        //             PPS-OUTLIER-THRESHOLD / P-NEW-OPER-CSTCHG-RATIO
        prospectivePaymentRecord.setChargeThreshold(prospectivePaymentRecord.getOutlierThreshold().divide(providerRecord.getCostToChargeRatio(), RoundingMode.HALF_UP));
        // TODO check with web pricer team to see if this is needed
        if (!calculationContext.getPcPricer()) {
          prospectivePaymentRecord.setRtc("67");
        }
      }
    }
    // setting to true for second loop through which happens regardless of payment type
    calculationContext.setCalculateOutliers(true);
  }

  private boolean isNonBlendNonSiteNeutralOutlierPaymentOrChangeOfTherapy(LtchPricerContext calculationContext, ProspectivePaymentRecord prospectivePaymentRecord, PaymentType paymentType) {
    // IF (PPS-OUTLIER-PAY-AMT > 0 AND
    //    NOT (PMT-BLEND AND H-OUTLIER-PAY-AMT-SNT = 0)) OR
    //   PPS-COT-IND = 'Y'
    return BigDecimalUtils.isGreaterThanZero(prospectivePaymentRecord.getOutlierPaymentAmount()) && !(paymentType.equals(PaymentType.BLEND) && BigDecimalUtils.isZero(calculationContext.getHoldOutlierPayAmountSiteNeutral())) || "Y".equals(prospectivePaymentRecord.getChangeOfTherapyIndicator());
  }

  private boolean isOutlierAndSuccessful(LtchPricerContext calculationContext, ProspectivePaymentRecord prospectivePaymentRecord) {
    return calculationContext.getCalculateOutliers() && !ErrorCode.isErrorCode(prospectivePaymentRecord.getRtc());
  }
}
