// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.codes.ErrorCode;
import gov.cms.fiss.pricers.ltch.core.codes.PaymentType;
import gov.cms.fiss.pricers.ltch.core.codes.SecondaryPaymentTypeSiteNeutral;
import gov.cms.fiss.pricers.ltch.core.codes.SecondaryPaymentTypeStandard;
import java.math.BigDecimal;
import java.math.RoundingMode;

public class CalculateHighCostOutlierBlend implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  private static final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(CalculateHighCostOutlierBlend.class);
  private static final String ONE = "1";

  /**
   * <pre>
   * *-------------------------------------------------------------*
   * * FOR BLENDED PAYMENT CLAIMS                                  *
   * * (RECEIVE 50% STANDARD PAYMENT + 50% SITE-NEUTRAL PAYMENT)   *
   * *-------------------------------------------------------------*
   * * - DETERMINE WHICH PAYMENT TO USE TO CALC OUTLIER THRESHOLDS *
   * *   BASED ON CLAIMS'S PAYMENT TYPE & PAYMENT FIELD VALUES     *
   * * - CALCULATE STANDARD PAYMENT OUTLIER THRESHOLD              *
   * * - CALCULATE SITE-NEUTRAL PAYMENT OUTLIER THRESHOLD          *
   * * - CALCULATE HIGH-COST OUTLIER IF APPLICABLE (50/50 BLEND)   *
   * *-------------------------------------------------------------*
   * </pre>
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    PaymentType paymentType = calculationContext.getPaymentType();
    SecondaryPaymentTypeStandard secondaryPaymentTypeStandard = calculationContext.getSecondaryPaymentTypeStandard();
    SecondaryPaymentTypeSiteNeutral secondaryPaymentTypeSiteNeutral = calculationContext.getSecondaryPaymentTypeSiteNeutral();
    ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    BillingRecord billingRecord = calculationContext.getBillingRecord();
    if (isOutlierAndSuccessful(calculationContext, prospectivePaymentRecord)) {
      if (paymentType.equals(PaymentType.BLEND)) {
        calculateStandardOutlierThreshold(calculationContext, secondaryPaymentTypeStandard, prospectivePaymentRecord);
        // *-------------------------------------------------------------*
        // * CALCULATE SITE-NEUTRAL OUTLIER THRESHOLD                    *
        // *-------------------------------------------------------------*
        if (secondaryPaymentTypeSiteNeutral.equals(SecondaryPaymentTypeSiteNeutral.IPPS)) {
          // COMPUTE PPS-OUTLIER-THRESHOLD ROUNDED =
          //                   PPS-SITE-NEUTRAL-IPPS-PMT +
          //                   H-FIXED-LOSS-AMT-SNT
          prospectivePaymentRecord.setOutlierThreshold(prospectivePaymentRecord.getPaymentData().getSiteNeutralIppsPayment().add(calculationContext.getHoldFixedLossAmountSiteNeutral()));
        }
        // *-------------------------------------------------------------*
        // * CALCULATE STANDARD PAYMENT PORTION OF HIGH-COST OUTLIER IF  *
        // * COSTS EXCEED STANDARD PAYMENT THRESHOLD                     *
        // *-------------------------------------------------------------*
        // IF PPS-FAC-COSTS > H-OUTLIER-THRESHOLD-STD
        if (BigDecimalUtils.isGreaterThan(prospectivePaymentRecord.getFacilityCosts(), calculationContext.getHoldOutlierThresholdStandard())) {
          // COMPUTE H-OUTLIER-PAY-AMT-STD ROUNDED =
          //                   (PPS-FAC-COSTS - H-OUTLIER-THRESHOLD-STD) * .8
          //                   * H-BLEND-STD
          calculationContext.setHoldOutlierPayAmountStandard(prospectivePaymentRecord.getFacilityCosts().subtract(calculationContext.getHoldOutlierThresholdStandard()).multiply(new BigDecimal("0.8")).multiply(calculationContext.getHoldBlendStandard()).setScale(2, RoundingMode.HALF_UP));
        }
        // *-------------------------------------------------------------*
        // * CALCULATE SITE-NEUTRAL PORTION OF HIGH-COST OUTLIER IF      *
        // * COSTS EXCEED THE SITE-NEUTRAL PAYMENT THRESHOLD             *
        // *-------------------------------------------------------------*
        // IF PMT-SITE-NEUT-IPPS AND
        //       PPS-FAC-COSTS > PPS-OUTLIER-THRESHOLD
        if (secondaryPaymentTypeSiteNeutral.equals(SecondaryPaymentTypeSiteNeutral.IPPS) && BigDecimalUtils.isGreaterThan(prospectivePaymentRecord.getFacilityCosts(), prospectivePaymentRecord.getOutlierThreshold())) {
          // COMPUTE H-OUTLIER-PAY-AMT-SNT ROUNDED =
          //                  (((PPS-FAC-COSTS - PPS-OUTLIER-THRESHOLD) * .8)
          //                   * H-BLEND-SNT)
          calculationContext.setHoldOutlierPayAmountSiteNeutral(prospectivePaymentRecord.getFacilityCosts().subtract(prospectivePaymentRecord.getOutlierThreshold()).multiply(new BigDecimal("0.8")).multiply(calculationContext.getHoldBlendSiteNeutral()).setScale(2, RoundingMode.HALF_UP));
        }
        // *-------------------------------------------------------------*
        // * CALCULATE TOTAL BLENDED HIGH-COST OUTLIER:                  *
        // * ADD THE SITE-NEUTRAL PAYMENT PORTION OF HIGH-COST OUTLIER   *
        // * TO THE STANDARD PAYMENT PORTION                             *
        // *-------------------------------------------------------------*
        // COMPUTE PPS-OUTLIER-PAY-AMT ROUNDED =
        //                H-OUTLIER-PAY-AMT-STD +
        //                (H-OUTLIER-PAY-AMT-SNT * H-SITE-NEUTRAL-IPPS-ADJ)
        prospectivePaymentRecord.setOutlierPaymentAmount(calculationContext.getHoldOutlierPayAmountSiteNeutral().multiply(calculationContext.getHoldSiteNeutralIppsAdj()).add(calculationContext.getHoldOutlierPayAmountStandard()).setScale(2, RoundingMode.HALF_UP));
      }
      // *-------------------------------------------------------------*
      // * FOR ALL CLAIMS:                                             *
      // *-------------------------------------------------------------*
      // * SET HIGH-COST OUTLIER TO $0 IF BILL SPECIAL PAY IND. = '1'  *
      // *-------------------------------------------------------------*
      // TODO move this into the non blended rule
      if (ONE.equals(billingRecord.getSpecialPaymentIndicator())) {
        prospectivePaymentRecord.setOutlierPaymentAmount(BigDecimal.ZERO);
      }
    }
  }

  private void calculateStandardOutlierThreshold(LtchPricerContext calculationContext, SecondaryPaymentTypeStandard secondaryPaymentTypeStandard, ProspectivePaymentRecord prospectivePaymentRecord) {
    // *-------------------------------------------------------------*
    // * CALCULATE STANDARD OUTLIER THRESHOLD                        *
    // *-------------------------------------------------------------*
    if (secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.FULL)) {
      // COMPUTE H-OUTLIER-THRESHOLD-STD ROUNDED =
      //                   PPS-STANDARD-FULL-PMT + H-FIXED-LOSS-AMT-STD
      calculationContext.setHoldOutlierThresholdStandard(prospectivePaymentRecord.getPaymentData().getStandardFullPayment().add(calculationContext.getHoldFixedLossAmountStandard()));
    } else {
      // COMPUTE H-OUTLIER-THRESHOLD-STD ROUNDED =
      //                   PPS-STANDARD-SSO-PMT + H-FIXED-LOSS-AMT-STD
      calculationContext.setHoldOutlierThresholdStandard(prospectivePaymentRecord.getPaymentData().getStandardShortStayOutliersPayment().add(calculationContext.getHoldFixedLossAmountStandard()));
    }
  }

  private boolean isOutlierAndSuccessful(LtchPricerContext calculationContext, ProspectivePaymentRecord prospectivePaymentRecord) {
    return calculationContext.getCalculateOutliers() && !ErrorCode.isErrorCode(prospectivePaymentRecord.getRtc());
  }
}
