// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * Calculates a short-stay blended payment if appropriate.
 *
 * <pre>
 * ***************************************************************
 * *    IF THE LENGTH OF STAY IS LESS THAN OR EQUAL TO 5/6       *
 * *      OF THE AVG. LENGTH OF STAY THEN:                       *
 * *      - PAY THE SHORT-STAY BLENDED PAYMENT                   *
 * *      - SET RETURN CODE FOR OLD POLICY CLAIMS ONLY TO        *
 * *        INDICATE SHORT STAY PAYMENT                          *
 * ***************************************************************
 * </pre>
 *
 * <p>Converted from {@code 3200-CALC-STANDARD-PMT} and {@code 3400-SHORT-STAY} in the COBOL code.
 */
public class CalculateShortStay implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  private static final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(CalculateShortStay.class);

  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    // IF H-LOS <= H-SSOT
    // IF PMT-STANDARD-NEW OR PMT-BLEND
    if ("00".equals(prospectivePaymentRecord.getRtc()) && calculationContext.isPaymentStandardOrBlend() && BigDecimalUtils.isLessThanOrEqualTo(calculationContext.getHoldLengthOfStay(), calculationContext.getHoldShortStayOutlierThreshold())) {
      // COMPUTE H-SS-PAY-AMT ROUNDED =
      //         ((PPS-DRG-ADJ-PAY-AMT / PPS-AVG-LOS) * H-LOS) * 1.2
      calculationContext.setHoldShortStayPayAmount(prospectivePaymentRecord.getAdjustedPaymentAmount().divide(prospectivePaymentRecord.getAverageLengthOfStay(), 3, RoundingMode.DOWN).multiply(calculationContext.getHoldLengthOfStay()).multiply(new BigDecimal("1.2")).setScale(2, RoundingMode.HALF_UP));
    }
  }
}
