// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.codes.SecondaryPaymentTypeSiteNeutral;
import java.math.BigDecimal;
import java.math.RoundingMode;

public class CalculateSiteNeutralPayment implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  private static final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(CalculateSiteNeutralPayment.class);

  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    final gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    if (calculationContext.isPaymentBlendOrSiteNeutral() && "00".equals(prospectivePaymentRecord.getRtc())) {
      // *-------------------------------------------------------------*
      // * CALCULATE SITE NEUTRAL IPPS COMP. PMT HIGH COST OUTLIER     *
      // * (HCO) FOR THE MINIMUM PAYMENT COMPARISON                    *
      // *-------------------------------------------------------------*
      // COMPUTE PPS-OUTLIER-PAY-AMT ROUNDED =
      //         PPS-OUTLIER-PAY-AMT *
      //         H-SITE-NEUTRAL-IPPS-ADJ.
      prospectivePaymentRecord.setOutlierPaymentAmount(prospectivePaymentRecord.getOutlierPaymentAmount().multiply(calculationContext.getHoldSiteNeutralIppsAdj()));
      calculationContext.setHoldOutlierIppsComparable(prospectivePaymentRecord.getOutlierPaymentAmount());
      prospectivePaymentRecord.getPaymentData().setSiteNeutralIppsPayment(BigDecimal.ZERO);
      prospectivePaymentRecord.setOutlierPaymentAmount(BigDecimal.ZERO);
      calculationContext.setHoldOutlierIppsComparable(prospectivePaymentRecord.getOutlierPaymentAmount());
      // *-------------------------------------------------------------*
      // * CALCULATE THE SITE NEUTRAL PAYMENT USING THE FOLLOWING      *
      // * FORMULA FROM POLICY GROUP:                                  *
      // * MIN ( SN ADJ * (IPPS PER DIEM AMT + SN IPPS HCO) , COST)    *
      // *-------------------------------------------------------------*
      // *-------------------------------------------------------------*
      // * CALCULATE SITE NEUTRAL COST PAYMENT FOR MIN PMT COMPARISON  *
      // *-------------------------------------------------------------*
      calculationContext.setHoldSnCostComparison(prospectivePaymentRecord.getFacilityCosts());
      // *-------------------------------------------------------------*
      // * CALCULATE SITE NEUTRAL IPPS COMP PMT FOR MIN PMT COMPARISON *
      // *-------------------------------------------------------------*
      // COMPUTE H-SN-IPPS-4COMPARISON ROUNDED =
      //         H-SITE-NEUTRAL-IPPS-ADJ *
      //         (H-IPPS-PER-DIEM +
      //         H-OUTLIER-IPPS-COMPARABLE).
      calculationContext.setHoldSnIppsComparison(calculationContext.getHoldIppsPerDiem().add(calculationContext.getHoldOutlierIppsComparable()).multiply(calculationContext.getHoldSiteNeutralIppsAdj()).setScale(2, RoundingMode.HALF_UP));
      // *-------------------------------------------------------------*
      // * MINIMUM IS FINAL SITE NEUTRAL PMT BASED ON COST             *
      // *-------------------------------------------------------------*
      // IF H-SN-COST-4COMPARISON < H-SN-IPPS-4COMPARISON
      if (BigDecimalUtils.isLessThan(calculationContext.getHoldSnCostComparison(), calculationContext.getHoldSnIppsComparison())) {
        calculationContext.setSecondaryPaymentTypeSiteNeutral(SecondaryPaymentTypeSiteNeutral.COST);
        prospectivePaymentRecord.getPaymentData().setSiteNeutralCostPayment(prospectivePaymentRecord.getFacilityCosts());
      } else {
        // *-------------------------------------------------------------*
        // * MINIMUM IS FINAL SITE-NEUTRAL PMT BASED ON IPPS COMPARABLE  *
        // *-------------------------------------------------------------*
        calculationContext.setSecondaryPaymentTypeSiteNeutral(SecondaryPaymentTypeSiteNeutral.IPPS);
        prospectivePaymentRecord.getPaymentData().setSiteNeutralIppsPayment(calculationContext.getHoldIppsPerDiem());
      }
    }
  }
}
