package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import java.math.BigDecimal;
import java.math.RoundingMode;

public class CapLtchWageIndexDecrease
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  /**
   *
   *
   * <pre>
   * *** SUPPLEMENTAL WAGE INDEX ***
   * * WHEN P-SUPP-WI-IND = '1' THE SUPPLEMENTAL WAGE INDEX FIELD
   * * CONTAINS THE PRIOR YEAR'S LTCH WAGE INDEX.
   * * THIS CALCULATION CAPS THE LTCH WAGE INDEX DECREASE AT 5%.
   * </pre>
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    // IF P-SUPP-WI-IND = '1'
    if ("1".equals(calculationContext.getSupplementalWageIndexIndicator())) {
      // IF P-SUPP-WI > 0
      if (BigDecimalUtils.isGreaterThanZero(calculationContext.getSupplementalWageIndex())) {
        // COMPUTE H-LTCH-SUPP-WI-RATIO =
        // (W-NEW-INDEX3-RECORD-C - P-SUPP-WI) / P-SUPP-WI
        BigDecimal supplementalWageIndexRatio =
            calculationContext
                .getLtchWageIndexTableEntry()
                .getLtchWageIndex3()
                .subtract(calculationContext.getSupplementalWageIndex())
                .divide(calculationContext.getSupplementalWageIndex(), RoundingMode.HALF_UP);
        // IF H-LTCH-SUPP-WI-RATIO <  -0.05
        if (BigDecimalUtils.isLessThan(supplementalWageIndexRatio, new BigDecimal("-0.05"))) {
          // TODO review and change name
          // COMPUTE W-NEW-INDEX3-RECORD-C ROUNDED =
          //        P-SUPP-WI * 0.95
          BigDecimal ltchWageIndexCap =
              calculationContext
                  .getProviderRecord()
                  .getCbsaData()
                  .getSupplementalWageIndex()
                  .multiply(new BigDecimal("0.95"))
                  .setScale(4, RoundingMode.HALF_UP);
          // TODO probably can delete the next line
          calculationContext.getLtchWageIndexTableEntry().setLtchWageIndex3(ltchWageIndexCap);
          calculationContext.getProspectivePaymentRecord().setWageIndex(ltchWageIndexCap);
        } else {
          calculationContext.getProspectivePaymentRecord().setRtc("52");
        }
      } else {
        calculationContext.getProspectivePaymentRecord().setRtc("52");
      }
    }
  }

  @Override
  public boolean shouldExecute(LtchPricerContext calculationContext) {
    return !"1"
        .equals(calculationContext.getProviderRecord().getCbsaData().getSpecialPaymentIndicator());
  }
}
