// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import org.apache.commons.lang3.StringUtils;

/**
 * <pre>
 * ******************************************************************
 * * GET LTCH WAGE INDEX
 * *----------------------------------------------------------------*
 * </pre>
 *
 * <p>Converted from {@code 0550-GET-CBSA}
 */
public class DetermineLtchWageIndex implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    final gov.cms.fiss.pricers.ltch.api.BillingRecord billingRecord = calculationContext.getBillingRecord();
    final gov.cms.fiss.pricers.ltch.api.ProviderRecord providerRecord = calculationContext.getProviderRecord();
    final gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    final java.time.LocalDate fyBegin = calculationContext.getFyBegin();
    final java.time.LocalDate fyEnd = calculationContext.getFyEnd();
    final gov.cms.fiss.pricers.ltch.core.models.LtchWageIndexTableEntry wageIndexTable = calculationContext.getLtchWageIndexTableEntry();
    final gov.cms.fiss.pricers.ltch.core.tables.DataTables dataTables = calculationContext.getDataTables();
    boolean specialWageIndexSet = false;
    if (calculationContext.isSpecialPaymentIndicatorSpecial(providerRecord.getCbsaData().getSpecialPaymentIndicator())) {
      // * USE SPECIAL WAGE INDEX WHEN INDICATED - FOR LTCH WAGE INDEX    *
      // * TO USE THE SPECIAL WAGE INDEX IT MUST:                         *
      // *   1) BE GREATER THAN 0, AND                                    *
      // *   2) BE IN A PSF RECORD WITH AN EFFECTIVE DATE WITHIN THE      *
      // *      CLAIM'S FISCAL YEAR.                                      *
      if (BigDecimalUtils.isGreaterThanZero(providerRecord.getCbsaData().getSpecialWageIndex()) && LocalDateUtils.inRange(providerRecord.getEffectiveDate(), fyBegin, fyEnd)) {
        // MOVE ZEROS                    TO W-NEW-CBSA
        // MOVE P-NEW-EFF-DATE           TO W-NEW-EFF-DATE-C
        // MOVE P-NEW-SPECIAL-WAGE-INDEX TO W-NEW-INDEX1-RECORD-C
        // MOVE P-NEW-SPECIAL-WAGE-INDEX TO W-NEW-INDEX2-RECORD-C
        // MOVE P-NEW-SPECIAL-WAGE-INDEX TO W-NEW-INDEX3-RECORD-C
        wageIndexTable.setCbsa(0);
        wageIndexTable.setEffectiveDate(providerRecord.getEffectiveDate());
        wageIndexTable.setLtchWageIndex1(providerRecord.getCbsaData().getSpecialWageIndex());
        wageIndexTable.setLtchWageIndex2(providerRecord.getCbsaData().getSpecialWageIndex());
        wageIndexTable.setLtchWageIndex3(providerRecord.getCbsaData().getSpecialWageIndex());
        specialWageIndexSet = true;
      } else {
        prospectivePaymentRecord.setRtc("52");
      }
    }
    // *6-20-16
    // *ADDED FOLLOWING LINE TO ALLOW LTCH PRICER TO WORK
    // *WITH THE NEW ARIZONA STATE CODE OF 00
    String trimmedGeolocation;
    if (providerRecord.getCbsaData().getGeolocation() != null) {
      trimmedGeolocation = providerRecord.getCbsaData().getGeolocation().trim();
    } else {
      trimmedGeolocation = "";
    }
    if ("00".equalsIgnoreCase(trimmedGeolocation)) {
      trimmedGeolocation = "03";
    }
    // * TO SELECT THE WAGE INDEX, IT MUST BE EITHER:                   *
    // * 1) AN INDIAN HEALTH CBSA (98 OR 99) WITH AN EFFECTIVE DATE ON  *
    // *    OR BEFORE THE CLAIM DISCHARGE DATE, OR                      *
    // * 2) A CBSA WITH AN EFFECTIVE DATE ON OR BEFORE THE CLAIM
    // *    DISCHARGE DATE AND A CLAIM DISCHARGE DATE ON OR BEFORE
    // *    09/30/2009 OR
    // * 3) A NON-INDIAN HEATLH CBSA WITH AN EFFECTIVE DATE ON OR BEFORE*
    // *    THE CLAIM DISHARGE DATE AND WITHIN THE CLAIM'S FISCAL YEAR  *
    final gov.cms.fiss.pricers.ltch.core.tables.CbsaTableEntry cbsaIndex = dataTables.getCbsaWageIndex(trimmedGeolocation, providerRecord.getEffectiveDate());
    if (!specialWageIndexSet && cbsaIndex != null && LocalDateUtils.isAfterOrEqual(billingRecord.getDischargeDate(), cbsaIndex.getEffectiveDate()) && (StringUtils.equalsAny(trimmedGeolocation, "98", "99") || LocalDateUtils.inRange(cbsaIndex.getEffectiveDate(), fyBegin, fyEnd))) {
      // MOVE CBSAX-CBSA (CU2)        TO W-NEW-CBSA
      // MOVE CBSAX-EFF-DATE (CU2)    TO W-NEW-EFF-DATE-C
      // MOVE CBSAX-WAGE-INDEX1 (CU2) TO W-NEW-INDEX1-RECORD-C
      // MOVE CBSAX-WAGE-INDEX2 (CU2) TO W-NEW-INDEX2-RECORD-C
      // MOVE CBSAX-WAGE-INDEX3 (CU2) TO W-NEW-INDEX3-RECORD-C
      wageIndexTable.setCbsa(Integer.valueOf(trimmedGeolocation));
      wageIndexTable.setEffectiveDate(cbsaIndex.getEffectiveDate());
      wageIndexTable.setLtchWageIndex1(cbsaIndex.getWageIndex1());
      wageIndexTable.setLtchWageIndex2(cbsaIndex.getWageIndex2());
      wageIndexTable.setLtchWageIndex3(cbsaIndex.getWageIndex3());
    } else if (cbsaIndex == null) {
      prospectivePaymentRecord.setRtc("60");
    }
    calculationContext.setProspectivePaymentRecord(prospectivePaymentRecord);
    calculationContext.setLtchWageIndexTableEntry(wageIndexTable);
  }
}
