// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.codes.PaymentType;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.Arrays;
import org.apache.commons.lang3.StringUtils;

public class DeterminePaymentType implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {
  /*   -------------------------------------------------------------*
      DETERMINE WHICH PAYMENT METHOD TO USE. EITHER:            *
      STANDARD PAYMENT UNDER THE OLD POLICY,                    *
      STANDARD PAYMENT UNDER THE NEW POLICY,                    *
      100% SITE NEUTRAL PAYMENT, OR                             *
      50/50 BLEND OF SITE NEUTRAL & STANDARD PAYMENT.           *
  *-------------------------------------------------------------*/
  /**
   * Converted from 3100-DETERMINE-PAYMENT-TYPE in COBOL.
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    PaymentType returnPaymentType = null;
    final gov.cms.fiss.pricers.ltch.api.BillingRecord billingRecord = calculationContext.getBillingRecord();
    final gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    final gov.cms.fiss.pricers.ltch.api.ProviderRecord providerRecord = calculationContext.getProviderRecord();
    if (StringUtils.equals(providerRecord.getCbsaData().getSupplementalWageIndexIndicator(), "2") && BigDecimalUtils.isGreaterThanZero(calculationContext.getSupplementalWageIndex())) {
      calculationContext.setHoldCapitalGeographicAreaFactor(BigDecimalUtils.pow(calculationContext.getSupplementalWageIndex(), new BigDecimal("0.6848"), 10));
    }
    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      prospectivePaymentRecord.setFacilityCosts(providerRecord.getCostToChargeRatio().multiply(billingRecord.getCoveredCharges()).setScale(2, RoundingMode.HALF_UP));
      int reviewCode = billingRecord.getReviewCode();
      // STANDARD PAYMENT (OLD POLICY)
      if (reviewCode == 0) {
        returnPaymentType = PaymentType.STANDARD_OLD;
      } else 
      // STANDARD PAYMENT (NEW POLICY)
      if (calculationContext.isStandardPaymentNew()) {
        returnPaymentType = PaymentType.STANDARD_NEW;
      } else 
      // *-------------------------------------------------------------*
      // * FOR COVID-19 PRICING, CLAIMS THAT WOULD NORMALLY GET PAID   *
      // * AS SITE-NEUTRAL WILL GET PAID AS STANDARD WHEN THE          *
      // * ADMISSION DATE IS ON OR AFTER JANUARY 27, 2020.             *
      // * SINCE THE ADMISSION DATE IS NOT INCLUDED IN THE INPUT BILL  *
      // * RECORD IT GETS CALCULATED FIRST.                            *
      // *-------------------------------------------------------------*
      if (reviewCode == 1 && prospectivePaymentRecord.isPsychRehabDrg() || Arrays.asList(2, 3, 6, 7, 8).contains(reviewCode)) {
        LocalDate admissionDate = billingRecord.getDischargeDate().minus(billingRecord.getLengthOfStay(), ChronoUnit.DAYS);
        if (LocalDateUtils.inRange(admissionDate, LocalDate.of(2020, 1, 26), false, calculationContext.getPublicHealthEmergencyLastDay(), true)) {
          calculationContext.setPaymentType(PaymentType.STANDARD_NEW);
          return;
        }
        // TODO refactor into pricer context
        calculationContext.setHoldBudgetNeutralFactor(new BigDecimal("0.949").setScale(6, RoundingMode.HALF_UP));
        calculationContext.setHoldSiteNeutralIppsAdj(new BigDecimal("0.954").setScale(6, RoundingMode.HALF_UP));
        returnPaymentType = getReturnPaymentType(calculationContext, prospectivePaymentRecord);
      }
      // *-------------------------------------------------------------*
      // * CLAIM MEETS NONE OF THE ABOVE CRITERIA - SET RETURN CODE
      // *-------------------------------------------------------------*
      if (returnPaymentType == null) {
        prospectivePaymentRecord.setRtc("72");
      }
      calculationContext.setPaymentType(returnPaymentType);
    }
  }

  private PaymentType getReturnPaymentType(LtchPricerContext calculationContext, ProspectivePaymentRecord prospectivePaymentRecord) {
    PaymentType returnPaymentType = null;
    if (prospectivePaymentRecord.getBlendYear() == 8) {
      // SITE NEUTRAL PAYMENT (NEW POLICY)
      returnPaymentType = PaymentType.SITE_NEUTRAL;
    } else if (prospectivePaymentRecord.getBlendYear() == 6 || prospectivePaymentRecord.getBlendYear() == 7) {
      // 50% SITE NEUTRAL + 50% STANDARD BLENDED PAYMENT
      returnPaymentType = PaymentType.BLEND;
      calculationContext.setHoldBlendStandard(new BigDecimal("0.5").setScale(1, RoundingMode.HALF_UP));
      calculationContext.setHoldBlendSiteNeutral(new BigDecimal("0.5").setScale(1, RoundingMode.HALF_UP));
    }
    return returnPaymentType;
  }
}
