package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchDrgsTableEntry;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;

public class EditDrgCode implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  /**
   * FINDS THE LTCH DRG CODE IN THE TABLE, sets the drg code, length of stay, and relative weight.
   * Sets return code 54 if it cannot be found.
   *
   * <p>Converted from {@code 1700-EDIT-DRG-CODE} in the COBOL code.
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    final BillingRecord billingRecord = calculationContext.getBillingRecord();
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      final String drgCode = billingRecord.getDiagnosisRelatedGroupCode();
      prospectivePaymentRecord.setSubmittedDrgCode(drgCode);
      final LtchDrgsTableEntry entry = calculationContext.getDataTables().getLtchDrgsEntry(drgCode);

      if (entry == null) {
        prospectivePaymentRecord.setRtc("54");
      } else {
        prospectivePaymentRecord.setRelativeWeight(entry.getRelativeWeight());
        prospectivePaymentRecord.setAverageLengthOfStay(entry.getArithmeticMeanLengthOfStay());
      }
    }
  }
}
