// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.api.ProviderRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import java.math.BigDecimal;
import java.time.LocalDate;

public class EditInputData implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {
  /**
   * Converted from 1000-EDIT-INPUT-DATA. in COBOL. Sets RTC code if any checks fail and does not
   * price
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    // EDIT BILL (BILL-NEW-DATA) INPUT & SET ERROR CODE IF FAIL
    final gov.cms.fiss.pricers.ltch.api.BillingRecord billingRecord = calculationContext.getBillingRecord();
    final gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      validateInputData(calculationContext);
    }
  }

  private void validateInputData(LtchPricerContext calculationContext) {
    final gov.cms.fiss.pricers.ltch.api.ProviderRecord providerRecord = calculationContext.getProviderRecord();
    final gov.cms.fiss.pricers.ltch.api.BillingRecord billingRecord = calculationContext.getBillingRecord();
    final gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    // TODO: Each check should see if the return code is 00 so it returns the first error code like
    // the COBOL
    if (billingRecord.getLengthOfStay() > 0) {
      calculationContext.setHoldLengthOfStay(BigDecimal.valueOf(billingRecord.getLengthOfStay()));
      prospectivePaymentRecord.setLengthOfStay(billingRecord.getLengthOfStay());
    } else {
      prospectivePaymentRecord.setRtc("56");
    }
    if (providerRecord.getCostOfLivingAdjustment() == null) {
      prospectivePaymentRecord.setRtc("50");
    }
    if ("Y".equals(providerRecord.getWaiverCode())) {
      prospectivePaymentRecord.setRtc("53");
    }
    if (billingRecord.getDiagnosisRelatedGroupCode() == null) {
      prospectivePaymentRecord.setRtc("54");
    }
    // TODO use localdate utils
    if (billingRecord.getDischargeDate().compareTo(providerRecord.getEffectiveDate()) < 0) {
      prospectivePaymentRecord.setRtc("55");
    }
    if (isProviderRecordTerminationDateInvalid(providerRecord, billingRecord)) {
      prospectivePaymentRecord.setRtc("51");
    }
    if (billingRecord.getCoveredCharges() == null) {
      prospectivePaymentRecord.setRtc("58");
    }
    if (isLifetimeReserveDaysInvalid(billingRecord)) {
      prospectivePaymentRecord.setRtc("61");
    }
    if (isCoveredDaysInvalid(prospectivePaymentRecord, billingRecord)) {
      prospectivePaymentRecord.setRtc("62");
    }
    if (isReviewCodeInvalid(billingRecord)) {
      prospectivePaymentRecord.setRtc("72");
    }
    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      // CALCULATE DAY RELATED VARIABLE VALUES
      calculationContext.setHoldRegularDays(billingRecord.getCoveredDays() - billingRecord.getLifetimeReserveDays());
      calculationContext.setHoldTotalDays(calculationContext.getHoldRegularDays() + billingRecord.getLifetimeReserveDays());
      daysUsed(calculationContext);
    }
    // *** -----------------------------------------------------------
    // *** EDIT PSF FIELDS USED BY ALL CLAIMS & SET ERROR CODE IF FAIL
    // *** -----------------------------------------------------------
    // *-------------------------------------------------------------*
    // * PROVIDER FY BEGIN DATE BEFORE THE FIRST PPS FEDERAL FY      *
    // * (ALWAYS FED-FY-BEGIN-03)                                    *
    // *-------------------------------------------------------------*
    // TODO use localdate utils
    if (calculationContext.getFyBegin().compareTo(LocalDate.of(2002, 10, 1)) < 0) {
      prospectivePaymentRecord.setRtc("74");
    }
    // *** -----------------------------------------------------------
    // *** EDITS FOR PSF FIELDS USED FOR THE 4TH SHORT STAY PROVISION
    // *** -----------------------------------------------------------
    if (providerRecord.getCapitalIndirectMedicalEducationRatio() != null) {
      calculationContext.setHoldCapitalTeachingAdjustmentRatio(providerRecord.getCapitalIndirectMedicalEducationRatio());
    } else {
      calculationContext.setHoldCapitalTeachingAdjustmentRatio(BigDecimal.ZERO);
    }
    calculationContext.setHoldInternRatio(providerRecord.getInternToBedRatio());
    calculationContext.setHoldBedSize(BigDecimal.valueOf(providerRecord.getBedSize()));
  }

  private boolean isProviderRecordTerminationDateInvalid(ProviderRecord providerRecord, BillingRecord billingRecord) {
    return providerRecord.getTerminationDate() != null && providerRecord.getTerminationDate().isBefore(billingRecord.getDischargeDate());
  }

  private boolean isLifetimeReserveDaysInvalid(BillingRecord billingRecord) {
    return billingRecord.getLifetimeReserveDays() == null || billingRecord.getLifetimeReserveDays() > 60;
  }

  private boolean isCoveredDaysInvalid(ProspectivePaymentRecord prospectivePaymentRecord, BillingRecord billingRecord) {
    return billingRecord.getCoveredDays() == null || billingRecord.getCoveredDays() == 0 && prospectivePaymentRecord.getLengthOfStay() > 0 || billingRecord.getLifetimeReserveDays() > billingRecord.getCoveredDays();
  }

  private boolean isReviewCodeInvalid(BillingRecord billingRecord) {
    return billingRecord.getReviewCode() == null || billingRecord.getReviewCode() < 0 || billingRecord.getReviewCode() > 8;
  }

  /**
   * Converted from 1200-DAYS-USED in COBOL. CALCULATE DAY RELATED VARIABLE VALUES
   */
  private void daysUsed(LtchPricerContext calculationContext) {
    final gov.cms.fiss.pricers.ltch.api.BillingRecord billingRecord = calculationContext.getBillingRecord();
    final gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord prospectivePaymentRecord = calculationContext.getProspectivePaymentRecord();
    int lifeTimeReserveDays = billingRecord.getLifetimeReserveDays();
    int coveredDays = billingRecord.getCoveredDays();
    int holdRegularDays = coveredDays - lifeTimeReserveDays;
    if (lifeTimeReserveDays > 0 && holdRegularDays == 0) {
      prospectivePaymentRecord.setLifetimeReserveDaysUsed(Math.min(lifeTimeReserveDays, calculationContext.getHoldLengthOfStay().intValue()));
    } else if (holdRegularDays > 0 && lifeTimeReserveDays == 0) {
      prospectivePaymentRecord.setRegularDaysUsed(Math.min(holdRegularDays, calculationContext.getHoldLengthOfStay().intValue()));
      prospectivePaymentRecord.setLifetimeReserveDaysUsed(0);
    } else if (holdRegularDays > 0 && lifeTimeReserveDays > 0) {
      if (holdRegularDays > calculationContext.getHoldLengthOfStay().intValue()) {
        prospectivePaymentRecord.setRegularDaysUsed(calculationContext.getHoldLengthOfStay().intValue());
        prospectivePaymentRecord.setLifetimeReserveDaysUsed(0);
      } else if (calculationContext.getHoldTotalDays() > calculationContext.getHoldLengthOfStay().intValue()) {
        prospectivePaymentRecord.setRegularDaysUsed(calculationContext.getHoldRegularDays());
        prospectivePaymentRecord.setLifetimeReserveDaysUsed(calculationContext.getHoldLengthOfStay().intValue() - calculationContext.getHoldRegularDays());
      } else if (calculationContext.getHoldTotalDays() <= calculationContext.getHoldLengthOfStay().intValue()) {
        prospectivePaymentRecord.setRegularDaysUsed(calculationContext.getHoldRegularDays());
        prospectivePaymentRecord.setLifetimeReserveDaysUsed(billingRecord.getLifetimeReserveDays());
      }
    }
  }
}
