package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.InpatientDrgsTableEntry;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;

public class EditIppsDrgCode implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  /**
   * Converted from 1800-EDIT-IPPS-DRG-CODE in COBOL. FINDS THE IPPS DRG CODE IN THE TABLE. sets
   * holder variables if needed
   */
  @Override
  public void priceClaim(LtchPricerContext ltchPricerContext) {
    // **-------------------------------------------------------**
    // ** THIS LOGIC WAS COPIED FROM THE IPPS PRICER (PPCAL140) **
    // ** ENSURE IT STAYS CONSISTENT BECAUSE IT REFERENCES THE  **
    // ** DRG TABLE USED BY THE IPPS PRICER.                    **
    // **-------------------------------------------------------**
    final BillingRecord billingRecord = ltchPricerContext.getBillingRecord();
    final ProspectivePaymentRecord prospectivePaymentRecord =
        ltchPricerContext.getProspectivePaymentRecord();

    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      final String drgCode = billingRecord.getDiagnosisRelatedGroupCode();
      final InpatientDrgsTableEntry entry =
          ltchPricerContext
              .getDataTables()
              .getInpatientDrgEntry(drgCode, billingRecord.getDischargeDate());
      if (entry == null || drgCode == null) {
        prospectivePaymentRecord.setRtc("54");
        ltchPricerContext.setProspectivePaymentRecord(prospectivePaymentRecord);
        return;
      }

      ltchPricerContext.setHoldIppsDrgWeight(entry.getWeight());
      ltchPricerContext.setHoldIppsDrgALengthOfStay(entry.getGeometricMeanLengthOfStay());
      ltchPricerContext.setHoldIppsArithALengthOfStay(entry.getArithmeticMeanLengthOfStay());
    }
  }
}
