package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.codes.ErrorCode;
import gov.cms.fiss.pricers.ltch.core.codes.PaymentType;
import java.math.BigDecimal;
import java.util.Arrays;

public class SetOldReturnCodes
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  /**
   *
   *
   * <pre>
   * ***************************************************************
   * *   SET RETURN CODES FOR OLD POLICY CLAIMS                    *
   * ***************************************************************
   * </pre>
   *
   * Converted from {@code 7100-SET-OLD-RETURN-CODES}
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    // TODO start from here next time
    PaymentType paymentType = calculationContext.getPaymentType();
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    if (!ErrorCode.isErrorCode(prospectivePaymentRecord.getRtc())
        && paymentType.equals(PaymentType.STANDARD_OLD)) {
      setReturnCodes(calculationContext);
    }
  }

  private void setReturnCodes(LtchPricerContext calculationContext) {
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    // *-------------------------------------------------------------*
    // * ALTER RETURN CODES FOR OLD POLICY CLAIMS TO REFLECT OUTLIER *
    // * PAYMENT IF OUTLIER PAYMENT IS > $0                          *
    // *-------------------------------------------------------------*
    if (BigDecimalUtils.isGreaterThanZero(prospectivePaymentRecord.getOutlierPaymentAmount())) {
      if ("21".equals(prospectivePaymentRecord.getRtc())) {
        prospectivePaymentRecord.setRtc("24");
      }
      if ("22".equals(prospectivePaymentRecord.getRtc())) {
        prospectivePaymentRecord.setRtc("25");
      }
      if ("26".equals(prospectivePaymentRecord.getRtc())) {
        prospectivePaymentRecord.setRtc("27");
      }
      if ("00".equals(prospectivePaymentRecord.getRtc())) {
        prospectivePaymentRecord.setRtc("01");
      }
    }

    if (Arrays.asList("00", "20", "21", "22", "26").contains(prospectivePaymentRecord.getRtc())
        && BigDecimalUtils.isGreaterThan(
            BigDecimal.valueOf(prospectivePaymentRecord.getRegularDaysUsed()),
            calculationContext.getHoldShortStayOutlierThreshold())) {
      prospectivePaymentRecord.setLifetimeReserveDaysUsed(0);
    }
  }
}
