package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import java.math.BigDecimal;
import java.time.LocalDate;

public class SupplementalWageIndexEdit
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  @Override
  public boolean shouldExecute(LtchPricerContext calculationContext) {
    // Only execute if the special wage index is not selected
    // TODO refactor to be readable
    return !"1"
        .equals(calculationContext.getProviderRecord().getCbsaData().getSpecialPaymentIndicator());
  }

  /**
   *
   *
   * <pre>
   * *----------------------------------------------------------*
   * * SUPPLEMENTAL WAGE INDEX EDIT                             *
   * *----------------------------------------------------------*
   * </pre>
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    String supplementalWageIndexIndicator =
        calculationContext.getProviderRecord().getCbsaData().getSupplementalWageIndexIndicator();
    BigDecimal supplementalWageIndex =
        calculationContext.getProviderRecord().getCbsaData().getSupplementalWageIndex();
    if (("1".equals(supplementalWageIndexIndicator) || "2".equals(supplementalWageIndexIndicator))
        && BigDecimalUtils.isLessThanOrEqualToZero(supplementalWageIndex)) {
      calculationContext.getProspectivePaymentRecord().setRtc("52");
      return;
    }
    LocalDate effectiveDate = calculationContext.getProviderRecord().getEffectiveDate();
    if (("1".equals(supplementalWageIndexIndicator) || "2".equals(supplementalWageIndexIndicator))
        // TODO use localdate utils
        && (effectiveDate.isBefore(calculationContext.getFyBegin())
            || effectiveDate.isAfter(calculationContext.getFyEnd()))) {
      calculationContext.getProspectivePaymentRecord().setRtc("52");
    }
  }
}
