package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;

public class SupplementalWageIndexToIppsWageIndex
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  @Override
  public boolean shouldExecute(LtchPricerContext calculationContext) {
    return !"1"
        .equals(calculationContext.getProviderRecord().getCbsaData().getSpecialPaymentIndicator());
  }

  /**
   *
   *
   * <pre>
   * * ADDED 9-10-20 FOR SUPPLEMENTAL WAGE INDEX.
   * * WHEN P-SUPP-WI-IND = '2' THE SUPPLEMENTAL WAGE INDEX FIELD
   * * CONTAINS THE IPPS WAGE INDEX VALUE.
   * </pre>
   *
   * For LTCHs a value of ‘2’ is the current year IPPS-comparable wage index value that is used for
   * short-stay outlier and site neutral payment rate payments.
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    if ("2".equals(calculationContext.getSupplementalWageIndexIndicator())) {
      if (BigDecimalUtils.isGreaterThanZero(calculationContext.getSupplementalWageIndex())) {
        calculationContext
            .getHoldProvIppsCBSA()
            .setWageIndex(calculationContext.getSupplementalWageIndex());
      } else {
        calculationContext.getProspectivePaymentRecord().setRtc("52");
      }
    }
  }
}
