// Generated by delombok at Tue Oct 12 15:40:24 EDT 2021
package gov.cms.fiss.pricers.ltch.core.tables;

import gov.cms.fiss.pricers.common.util.csv.CsvIngestionConfiguration;
import gov.cms.fiss.pricers.ltch.LtchPricerConfiguration;
import gov.cms.fiss.pricers.ltch.api.InpatientDrgsTableEntry;
import gov.cms.fiss.pricers.ltch.api.LtchDrgsTableEntry;
import java.time.LocalDate;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NavigableMap;
import java.util.concurrent.ConcurrentHashMap;
import org.apache.commons.lang3.StringUtils;

/**
 * This class is responsible for holding all the CopyBook data tables for the pricer as well as
 * helper methods to access the data.
 */
public class DataTables {
  private static final Map<Integer, DataTables> YEAR_LOOKUP = new ConcurrentHashMap<>();
  private final Map<String, NavigableMap<LocalDate, CbsaTableEntry>> cbsaMap;
  private final Map<String, NavigableMap<LocalDate, WageIppsIndexRuralEntry>> wageIppsIndexRuralMap;
  private final Map<String, NavigableMap<LocalDate, IppsCbsaTableEntry>> ippsCbsaMap;
  private final Map<String, NavigableMap<LocalDate, InpatientDrgsTableEntry>> ippsDrgsMap;
  private final Map<String, LtchDrgsTableEntry> ltchDrgsMap;

  public DataTables(Map<String, NavigableMap<LocalDate, CbsaTableEntry>> cbsaMap, Map<String, NavigableMap<LocalDate, WageIppsIndexRuralEntry>> wageIppsIndexRuralMap, Map<String, NavigableMap<LocalDate, IppsCbsaTableEntry>> ippsCbsaMap, Map<String, NavigableMap<LocalDate, InpatientDrgsTableEntry>> ippsDrgsMap, Map<String, LtchDrgsTableEntry> ltchDrgsMap) {
    this.cbsaMap = cbsaMap;
    this.wageIppsIndexRuralMap = wageIppsIndexRuralMap;
    this.ippsCbsaMap = ippsCbsaMap;
    this.ippsDrgsMap = ippsDrgsMap;
    this.ltchDrgsMap = ltchDrgsMap;
  }

  public static DataTables forYear(int pricerYear) {
    return YEAR_LOOKUP.get(pricerYear);
  }

  /**
   * Loads data tables with the given CSV ingestion configuration.
   */
  public static void loadDataTables(LtchPricerConfiguration pricerConfiguration) {
    final CsvIngestionConfiguration csvIngestionConfiguration = pricerConfiguration.getCsvIngestionConfiguration();
    final CbsaTableLookupGenerator cbsaTableLookupGenerator = new CbsaTableLookupGenerator(csvIngestionConfiguration);
    final WageIppsIndexRuralLookupGenerator wageIppsIndexRuralLookupGenerator = new WageIppsIndexRuralLookupGenerator(csvIngestionConfiguration);
    final IppsCbsaTableLookupGenerator ippsCbsaTableLookupGenerator = new IppsCbsaTableLookupGenerator(csvIngestionConfiguration);
    final InpatientDrgsTableLookupGenerator inpatientDrgsTableLookupGenerator = new InpatientDrgsTableLookupGenerator();
    final LtchDrgsTableLookupGenerator ltchDrgsTableLookupGenerator = new LtchDrgsTableLookupGenerator();
    // Load all the tables
    for (final int pricerYear : pricerConfiguration.getSupportedYears()) {
      final Map<String, NavigableMap<LocalDate, CbsaTableEntry>> cbsaMap = cbsaTableLookupGenerator.generate(pricerYear);
      final Map<String, NavigableMap<LocalDate, WageIppsIndexRuralEntry>> wageIppsIndexRuralMap = wageIppsIndexRuralLookupGenerator.generate();
      final Map<String, NavigableMap<LocalDate, IppsCbsaTableEntry>> ippsCbsaMap = ippsCbsaTableLookupGenerator.generate(pricerYear);
      final Map<String, NavigableMap<LocalDate, InpatientDrgsTableEntry>> ippsDrgsMap = inpatientDrgsTableLookupGenerator.generate(pricerYear);
      final Map<String, LtchDrgsTableEntry> ltchDrgsMap = ltchDrgsTableLookupGenerator.generate(pricerYear);
      YEAR_LOOKUP.put(pricerYear, new DataTables(cbsaMap, wageIppsIndexRuralMap, ippsCbsaMap, ippsDrgsMap, ltchDrgsMap));
    }
  }

  public WageIppsIndexRuralEntry getWageIppsIndexRural(String cbsa, LocalDate effectiveDate) {
    if (cbsa == null) {
      return null;
    }
    final NavigableMap<LocalDate, WageIppsIndexRuralEntry> entries = this.wageIppsIndexRuralMap.get(cbsa.trim());
    return (WageIppsIndexRuralEntry) getEntry(effectiveDate, entries);
  }

  public CbsaTableEntry getCbsaWageIndex(String cbsa, LocalDate effectiveDate) {
    final NavigableMap<LocalDate, CbsaTableEntry> entries = this.cbsaMap.get(cbsa);
    return (CbsaTableEntry) getEntry(effectiveDate, entries);
  }

  public IppsCbsaTableEntry getIppsCbsaWageIndex(String cbsa, LocalDate effectiveDate) {
    final NavigableMap<LocalDate, IppsCbsaTableEntry> entries = this.ippsCbsaMap.get(cbsa.trim());
    return (IppsCbsaTableEntry) getEntry(effectiveDate, entries);
  }

  private Object getEntry(LocalDate effectiveDate, NavigableMap<LocalDate, ?> map) {
    if (map == null) {
      return null;
    }
    final Entry<LocalDate, ?> floorEntry = map.floorEntry(effectiveDate);
    if (floorEntry == null) {
      return null;
    } else {
      return floorEntry.getValue();
    }
  }

  public InpatientDrgsTableEntry getInpatientDrgEntry(String code, LocalDate effectiveDate) {
    final NavigableMap<LocalDate, InpatientDrgsTableEntry> entries = ippsDrgsMap.get(StringUtils.leftPad(code, 3, "0"));
    if (entries != null) {
      final Entry<LocalDate, InpatientDrgsTableEntry> entry = entries.floorEntry(effectiveDate);
      return null == entry ? null : entry.getValue();
    }
    return null;
  }

  public LtchDrgsTableEntry getLtchDrgsEntry(String code) {
    return ltchDrgsMap.get(code);
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public Map<String, NavigableMap<LocalDate, InpatientDrgsTableEntry>> getIppsDrgsMap() {
    return this.ippsDrgsMap;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public Map<String, LtchDrgsTableEntry> getLtchDrgsMap() {
    return this.ltchDrgsMap;
  }
}
