package gov.cms.fiss.pricers.ltch.core.tables;

import com.fasterxml.jackson.dataformat.csv.CsvParser.Feature;
import com.fasterxml.jackson.dataformat.csv.CsvSchema;
import gov.cms.fiss.pricers.common.util.csv.CsvContentReader;
import gov.cms.fiss.pricers.common.util.csv.CsvIngestionConfiguration;
import gov.cms.fiss.pricers.common.util.csv.LookupGenerator;
import java.time.LocalDate;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.NavigableMap;
import java.util.TreeMap;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.stream.Collector;
import java.util.stream.Collectors;

public class IppsCbsaTableLookupGenerator {

  private static final String CSV_FILE_PATTERN = "/ipwixtable-%d.csv";

  private final CsvContentReader<IppsCbsaTableEntry> contentProvider =
      new CsvContentReader<>(IppsCbsaTableEntry.class);

  public IppsCbsaTableLookupGenerator(CsvIngestionConfiguration csvIngestionConfiguration) {
    if (!csvIngestionConfiguration.isValidationEnabled()) {
      contentProvider.disableValidation();
    }
    contentProvider.customizeMapper(mapper -> mapper.disable(Feature.FAIL_ON_MISSING_COLUMNS));
    contentProvider.withSchema(CsvSchema.emptySchema().withHeader());
  }

  public Map<String, NavigableMap<LocalDate, IppsCbsaTableEntry>> generate(int pricerYear) {
    final List<IppsCbsaTableEntry> content =
        contentProvider.read(String.format(CSV_FILE_PATTERN, pricerYear));
    final BinaryOperator<IppsCbsaTableEntry> comparator = BinaryOperator.maxBy((a, b) -> 0);
    final Collector<IppsCbsaTableEntry, ?, NavigableMap<LocalDate, IppsCbsaTableEntry>> collector =
        Collectors.toMap(
            IppsCbsaTableEntry::getEffectiveDate, Function.identity(), comparator, TreeMap::new);

    return LookupGenerator.generateCrossReferenceToMap(
        IppsCbsaTableEntry::getCbsa, content, HashMap::new, collector);
  }
}
